@@ -1,6 +1,6 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# pyre-ignore-all-errors[2,3]
 
-import copy
 import logging
 import os
 from collections import defaultdict
@@ -9,7 +9,7 @@ from typing import Any, Dict, Iterable, List, NamedTuple, Optional, Tuple
 import numpy as np
 import torch
 import torch.nn as nn
-from fvcore.common.file_io import PathManager
+from iopath.common.file_io import HTTPURLHandler, PathManager
 from termcolor import colored
 from torch.nn.parallel import DataParallel, DistributedDataParallel
 
@@ -17,21 +17,23 @@ from torch.nn.parallel import DataParallel, DistributedDataParallel
 __all__ = ["Checkpointer", "PeriodicCheckpointer"]
 
 
+TORCH_VERSION: Tuple[int, ...] = tuple(int(x) for x in torch.__version__.split(".")[:2])
+
+
 class _IncompatibleKeys(
     NamedTuple(
-        # pyre-fixme[10]: Name `IncompatibleKeys` is used but not defined.
         "IncompatibleKeys",
         [
             ("missing_keys", List[str]),
             ("unexpected_keys", List[str]),
-            ("incorrect_shapes", List[Tuple]),
+            ("incorrect_shapes", List[Tuple[str, Tuple[int], Tuple[int]]]),
         ],
     )
 ):
     pass
 
 
-class Checkpointer(object):
+class Checkpointer:
     """
     A checkpointer that can save/load model as well as extra checkpointable
     objects.
@@ -43,7 +45,7 @@ class Checkpointer(object):
         save_dir: str = "",
         *,
         save_to_disk: bool = True,
-        **checkpointables: object,
+        **checkpointables: Any,
     ) -> None:
         """
         Args:
@@ -52,19 +54,42 @@ class Checkpointer(object):
             save_to_disk (bool): if True, save checkpoint to disk, otherwise
                 disable saving for this checkpointer.
             checkpointables (object): any checkpointable objects, i.e., objects
-                that have the `state_dict()` and `load_state_dict()` method. For
+                that have the ``state_dict()`` and ``load_state_dict()`` method. For
                 example, it can be used like
                 `Checkpointer(model, "dir", optimizer=optimizer)`.
         """
         if isinstance(model, (DistributedDataParallel, DataParallel)):
             model = model.module
         self.model = model
-        self.checkpointables = copy.copy(checkpointables)  # pyre-ignore
-        self.logger = logging.getLogger(__name__)  # pyre-ignore
+        self.checkpointables: Dict[str, Any] = {}
+        for k, v in checkpointables.items():
+            self.add_checkpointable(k, v)
+        self.logger: logging.Logger = logging.getLogger(__name__)
         self.save_dir = save_dir
         self.save_to_disk = save_to_disk
+        # Default PathManager, support HTTP URLs (for backward compatibility in open source).
+        # A user may want to use a different project-specific PathManager
+        self.path_manager: PathManager = PathManager()
+        self.path_manager.register_handler(HTTPURLHandler())
 
-    def save(self, name: str, **kwargs: Dict[str, str]) -> None:
+    def add_checkpointable(self, key: str, checkpointable: Any) -> None:
+        """
+        Add checkpointable object for this checkpointer to track.
+
+        Args:
+            key (str): the key used to save the object
+            checkpointable: any object with ``state_dict()`` and
+                ``load_state_dict()`` method
+        """
+        if key in self.checkpointables:
+            raise KeyError(f"Key {key} already used in the Checkpointer")
+        if not hasattr(checkpointable, "state_dict"):
+            raise TypeError(
+                "add_checkpointable needs an object with 'state_dict()' method."
+            )
+        self.checkpointables[key] = checkpointable
+
+    def save(self, name: str, **kwargs: Any) -> None:
         """
         Dump model and checkpointables to a file.
 
@@ -85,14 +110,15 @@ class Checkpointer(object):
         save_file = os.path.join(self.save_dir, basename)
         assert os.path.basename(save_file) == basename, basename
         self.logger.info("Saving checkpoint to {}".format(save_file))
-        with PathManager.open(save_file, "wb") as f:
+        with self.path_manager.open(save_file, "wb") as f:
             torch.save(data, f)
         self.tag_last_checkpoint(basename)
 
-    def load(self, path: str, checkpointables: Optional[List[str]] = None) -> object:
+    def load(
+        self, path: str, checkpointables: Optional[List[str]] = None
+    ) -> Dict[str, Any]:
         """
-        Load from the given checkpoint. When path points to network file, this
-        function has to be called on all ranks.
+        Load from the given checkpoint.
 
         Args:
             path (str): path or url to the checkpoint. If empty, will not load
@@ -111,7 +137,7 @@ class Checkpointer(object):
             return {}
         self.logger.info("Loading checkpoint from {}".format(path))
         if not os.path.isfile(path):
-            path = PathManager.get_local_path(path)
+            path = self.path_manager.get_local_path(path)
             assert os.path.isfile(path), "Checkpoint {} not found!".format(path)
 
         checkpoint = self._load_file(path)
@@ -122,10 +148,10 @@ class Checkpointer(object):
             self._log_incompatible_keys(incompatible)
 
         for key in self.checkpointables if checkpointables is None else checkpointables:
-            if key in checkpoint:  # pyre-ignore
+            if key in checkpoint:
                 self.logger.info("Loading {} from {}".format(key, path))
                 obj = self.checkpointables[key]
-                obj.load_state_dict(checkpoint.pop(key))  # pyre-ignore
+                obj.load_state_dict(checkpoint.pop(key))
 
         # return any further checkpoint data
         return checkpoint
@@ -136,7 +162,7 @@ class Checkpointer(object):
             bool: whether a checkpoint exists in the target directory.
         """
         save_file = os.path.join(self.save_dir, "last_checkpoint")
-        return PathManager.exists(save_file)
+        return self.path_manager.exists(save_file)
 
     def get_checkpoint_file(self) -> str:
         """
@@ -145,13 +171,13 @@ class Checkpointer(object):
         """
         save_file = os.path.join(self.save_dir, "last_checkpoint")
         try:
-            with PathManager.open(save_file, "r") as f:
+            with self.path_manager.open(save_file, "r") as f:
                 last_saved = f.read().strip()
         except IOError:
             # if file doesn't exist, maybe because it has just been
             # deleted by a separate process
             return ""
-        return os.path.join(self.save_dir, last_saved)  # pyre-ignore
+        return os.path.join(self.save_dir, last_saved)  # pyre-fixme[6]
 
     def get_all_checkpoint_files(self) -> List[str]:
         """
@@ -161,13 +187,13 @@ class Checkpointer(object):
         """
         all_model_checkpoints = [
             os.path.join(self.save_dir, file)
-            for file in PathManager.ls(self.save_dir)
-            if PathManager.isfile(os.path.join(self.save_dir, file))
+            for file in self.path_manager.ls(self.save_dir)
+            if self.path_manager.isfile(os.path.join(self.save_dir, file))
             and file.endswith(".pth")
         ]
         return all_model_checkpoints
 
-    def resume_or_load(self, path: str, *, resume: bool = True) -> object:
+    def resume_or_load(self, path: str, *, resume: bool = True) -> Dict[str, Any]:
         """
         If `resume` is True, this method attempts to resume from the last
         checkpoint, if exists. Otherwise, load checkpoint from the given path.
@@ -196,13 +222,14 @@ class Checkpointer(object):
             last_filename_basename (str): the basename of the last filename.
         """
         save_file = os.path.join(self.save_dir, "last_checkpoint")
-        with PathManager.open(save_file, "w") as f:
+        with self.path_manager.open(save_file, "w") as f:
             f.write(last_filename_basename)  # pyre-ignore
 
-    def _load_file(self, f: str) -> object:
+    def _load_file(self, f: str) -> Dict[str, Any]:
         """
         Load a checkpoint file. Can be overwritten by subclasses to support
         different formats.
+
         Args:
             f (str): a locally mounted file path.
         Returns:
@@ -212,7 +239,7 @@ class Checkpointer(object):
         """
         return torch.load(f, map_location=torch.device("cpu"))
 
-    def _load_model(self, checkpoint: Any) -> _IncompatibleKeys:  # pyre-ignore
+    def _load_model(self, checkpoint: Any) -> _IncompatibleKeys:
         """
         Load weights from a checkpoint.
 
@@ -238,14 +265,50 @@ class Checkpointer(object):
         # remove the "module" prefix before performing the matching.
         _strip_prefix_if_present(checkpoint_state_dict, "module.")
 
-        # work around https://github.com/pytorch/pytorch/issues/24139
+        # workaround https://github.com/pytorch/pytorch/issues/24139
         model_state_dict = self.model.state_dict()
         incorrect_shapes = []
         for k in list(checkpoint_state_dict.keys()):
             if k in model_state_dict:
-                shape_model = tuple(model_state_dict[k].shape)
+                model_param = model_state_dict[k]
+                # Allow mismatch for uninitialized parameters
+                if TORCH_VERSION >= (1, 8) and isinstance(
+                    model_param, nn.parameter.UninitializedParameter
+                ):
+                    continue
+                shape_model = tuple(model_param.shape)
                 shape_checkpoint = tuple(checkpoint_state_dict[k].shape)
                 if shape_model != shape_checkpoint:
+
+                    has_observer_base_classes = (
+                        TORCH_VERSION >= (1, 8)
+                        and hasattr(torch.quantization, "ObserverBase")
+                        and hasattr(torch.quantization, "FakeQuantizeBase")
+                    )
+                    if has_observer_base_classes:
+                        # Handle the special case of quantization per channel observers,
+                        # where buffer shape mismatches are expected.
+                        def _get_module_for_key(
+                            model: torch.nn.Module, key: str
+                        ) -> torch.nn.Module:
+                            # foo.bar.param_or_buffer_name -> [foo, bar]
+                            key_parts = key.split(".")[:-1]
+                            cur_module = model
+                            for key_part in key_parts:
+                                cur_module = getattr(cur_module, key_part)
+                            return cur_module
+
+                        cls_to_skip = (
+                            torch.quantization.ObserverBase,
+                            torch.quantization.FakeQuantizeBase,
+                        )
+                        target_module = _get_module_for_key(self.model, k)
+                        if isinstance(target_module, cls_to_skip):
+                            # Do not remove modules with expected shape mismatches
+                            # them from the state_dict loading. They have special logic
+                            # in _load_from_state_dict to handle the mismatches.
+                            continue
+
                     incorrect_shapes.append((k, shape_checkpoint, shape_model))
                     checkpoint_state_dict.pop(k)
         # pyre-ignore
@@ -304,24 +367,28 @@ class PeriodicCheckpointer:
     Save checkpoints periodically. When `.step(iteration)` is called, it will
     execute `checkpointer.save` on the given checkpointer, if iteration is a
     multiple of period or if `max_iter` is reached.
+
+    Attributes:
+        checkpointer (Checkpointer): the underlying checkpointer object
     """
 
     def __init__(
         self,
-        checkpointer: Any,  # pyre-ignore
+        checkpointer: Checkpointer,
         period: int,
         max_iter: Optional[int] = None,
         max_to_keep: Optional[int] = None,
+        file_prefix: str = "model",
     ) -> None:
         """
         Args:
-            checkpointer (Any): the checkpointer object used to save
-            checkpoints.
+            checkpointer: the checkpointer object used to save checkpoints.
             period (int): the period to save checkpoint.
             max_iter (int): maximum number of iterations. When it is reached,
-                a checkpoint named "model_final" will be saved.
+                a checkpoint named "{file_prefix}_final" will be saved.
             max_to_keep (int): maximum number of most current checkpoints to keep,
                 previous checkpoints will be deleted
+            file_prefix (str): the prefix of checkpoint's filename
         """
         self.checkpointer = checkpointer
         self.period = int(period)
@@ -329,7 +396,9 @@ class PeriodicCheckpointer:
         if max_to_keep is not None:
             assert max_to_keep > 0
         self.max_to_keep = max_to_keep
-        self.recent_checkpoints = []  # pyre-ignore
+        self.recent_checkpoints: List[str] = []
+        self.path_manager: PathManager = checkpointer.path_manager
+        self.file_prefix = file_prefix
 
     def step(self, iteration: int, **kwargs: Any) -> None:
         """
@@ -343,22 +412,27 @@ class PeriodicCheckpointer:
         iteration = int(iteration)
         additional_state = {"iteration": iteration}
         additional_state.update(kwargs)
+
         if (iteration + 1) % self.period == 0:
-            self.checkpointer.save("model_{:07d}".format(iteration), **additional_state)
+            self.checkpointer.save(
+                "{}_{:07d}".format(self.file_prefix, iteration), **additional_state
+            )
 
             if self.max_to_keep is not None:
                 self.recent_checkpoints.append(self.checkpointer.get_checkpoint_file())
-                # pyre-fixme[6]: Expected `int` for 1st param but got `Optional[int]`.
-                # pyre-fixme[6]: Expected `int` for 1st param but got `Optional[int]`.
+                # pyre-fixme[58]: `>` is not supported for operand types `int` and
+                #  `Optional[int]`.
                 if len(self.recent_checkpoints) > self.max_to_keep:
                     file_to_delete = self.recent_checkpoints.pop(0)
-                    if PathManager.exists(
+                    if self.path_manager.exists(
                         file_to_delete
-                    ) and not file_to_delete.endswith("model_final.pth"):
-                        PathManager.rm(file_to_delete)
+                    ) and not file_to_delete.endswith(f"{self.file_prefix}_final.pth"):
+                        self.path_manager.rm(file_to_delete)
 
-        if iteration >= self.max_iter - 1:  # pyre-ignore
-            self.checkpointer.save("model_final", **additional_state)
+        if self.max_iter is not None:
+            # pyre-fixme[58]
+            if iteration >= self.max_iter - 1:
+                self.checkpointer.save(f"{self.file_prefix}_final", **additional_state)
 
     def save(self, name: str, **kwargs: Any) -> None:
         """
