@@ -328,6 +328,7 @@ class NativePathHandler(PathHandler):
         errors: Optional[str] = None,
         newline: Optional[str] = None,
         closefd: bool = True,
+        # pyre-fixme[24]: Generic type `Callable` expects 2 type parameters.
         opener: Optional[Callable] = None,
         **kwargs: Any,
     ) -> Union[IO[str], IO[bytes]]:
@@ -494,7 +495,6 @@ class HTTPURLHandler(PathHandler):
             cached = os.path.join(dirname, filename)
             with file_lock(cached):
                 if not os.path.isfile(cached):
-                    logger.info("Downloading {} ...".format(path))
                     cached = download(path, dirname, filename=filename)
             logger.info("URL {} cached in {}".format(path, cached))
             self.cache_map[path] = cached
@@ -825,16 +825,19 @@ class PathManagerBase:
 
             old_handler_type = type(self._path_handlers[prefix])
             if allow_override:
-                logger.warning(
-                    f"[PathManager] Attempting to register prefix '{prefix}' from "
-                    "the following call stack:\n" + "".join(traceback.format_stack())
-                )
-                logger.warning(
-                    f"[PathManager] Prefix '{prefix}' is already registered "
-                    f"by {old_handler_type}. We will override the old handler. "
-                    "To avoid such conflicts, create a project-specific PathManager "
-                    "instead."
-                )
+                # if using the global PathManager, show the warnings
+                if self == PathManager:
+                    logger.warning(
+                        f"[PathManager] Attempting to register prefix '{prefix}' from "
+                        "the following call stack:\n"
+                        + "".join(traceback.format_stack(limit=-5))
+                    )
+                    logger.warning(
+                        f"[PathManager] Prefix '{prefix}' is already registered "
+                        f"by {old_handler_type}. We will override the old handler. "
+                        "To avoid such conflicts, create a project-specific PathManager "
+                        "instead."
+                    )
                 self._path_handlers[prefix] = handler
             else:
                 raise KeyError(
