@@ -1,16 +1,18 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# pyre-ignore-all-errors[2,33]
 
 import logging
-import typing
 from collections import defaultdict
+from typing import Any, Callable, Counter, DefaultDict, Dict, List, Tuple, Union
 
 import torch.nn as nn
 
 from .jit_handles import generic_activation_jit, get_jit_model_analysis
 
 
+Handle = Callable[[List[Any], List[Any]], Counter[str]]
 # A dictionary that maps supported operations to their activation count handles.
-_DEFAULT_SUPPORTED_OPS: typing.Dict[str, typing.Callable] = {
+_DEFAULT_SUPPORTED_OPS: Dict[str, Handle] = {
     "aten::_convolution": generic_activation_jit("conv"),
     "aten::addmm": generic_activation_jit("addmm"),
 }
@@ -18,9 +20,9 @@ _DEFAULT_SUPPORTED_OPS: typing.Dict[str, typing.Callable] = {
 
 def activation_count(
     model: nn.Module,
-    inputs: typing.Tuple[object, ...],
-    supported_ops: typing.Union[typing.Dict[str, typing.Callable], None] = None,
-) -> typing.Tuple[typing.DefaultDict[str, float], typing.Counter[str]]:
+    inputs: Tuple[Any, ...],
+    supported_ops: Union[Dict[str, Handle], None] = None,
+) -> Tuple[DefaultDict[str, float], Counter[str]]:
     """
     Given a model and an input to the model, compute the total number of
     activations of the model.
