@@ -1,13 +1,15 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
+# pyre-ignore-all-errors[2,33]
 
 import logging
-import typing
 from collections import defaultdict
+from typing import Any, Callable, Counter, DefaultDict, Dict, List, Tuple, Union
 
 import torch.nn as nn
 
 from .jit_handles import (
     addmm_flop_jit,
+    bmm_flop_jit,
     conv_flop_jit,
     einsum_flop_jit,
     get_jit_model_analysis,
@@ -15,9 +17,11 @@ from .jit_handles import (
 )
 
 
+Handle = Callable[[List[Any], List[Any]], Counter[str]]
 # A dictionary that maps supported operations to their flop count jit handles.
-_DEFAULT_SUPPORTED_OPS: typing.Dict[str, typing.Callable] = {
+_DEFAULT_SUPPORTED_OPS: Dict[str, Handle] = {
     "aten::addmm": addmm_flop_jit,
+    "aten::bmm": bmm_flop_jit,
     "aten::_convolution": conv_flop_jit,
     "aten::einsum": einsum_flop_jit,
     "aten::matmul": matmul_flop_jit,
@@ -26,9 +30,9 @@ _DEFAULT_SUPPORTED_OPS: typing.Dict[str, typing.Callable] = {
 
 def flop_count(
     model: nn.Module,
-    inputs: typing.Tuple[object, ...],
-    supported_ops: typing.Union[typing.Dict[str, typing.Callable], None] = None,
-) -> typing.Tuple[typing.DefaultDict[str, float], typing.Counter[str]]:
+    inputs: Tuple[Any, ...],
+    supported_ops: Union[Dict[str, Handle], None] = None,
+) -> Tuple[DefaultDict[str, float], Counter[str]]:
     """
     Given a model and an input to the model, compute the Gflops of the given
     model.
