@@ -66,6 +66,7 @@ _IGNORED_OPS: typing.List[str] = [
 def get_jit_model_analysis(
     model: nn.Module,
     inputs: typing.Tuple[object, ...],
+    # pyre-fixme[24]: Generic type `typing.Callable` expects 2 type parameters.
     ops_handles: typing.Dict[str, typing.Callable],
 ) -> typing.Tuple[typing.Counter[str], typing.Counter[str]]:
     """
@@ -88,7 +89,7 @@ def get_jit_model_analysis(
     if isinstance(
         model, (nn.parallel.distributed.DistributedDataParallel, nn.DataParallel)
     ):
-        model = model.module  # pyre-ignore
+        model = model.module
 
     # Compatibility with torch.jit.
     if hasattr(torch.jit, "get_trace_graph"):
@@ -237,11 +238,12 @@ def conv_flop_jit(
         Counter: A Counter dictionary that records the number of flops for each
             operation.
     """
-    # Inputs of Convolution should be a list of length 12. They represent:
+    # Inputs of Convolution should be a list of length 12 or 13. They represent:
     # 0) input tensor, 1) convolution filter, 2) bias, 3) stride, 4) padding,
     # 5) dilation, 6) transposed, 7) out_pad, 8) groups, 9) benchmark_cudnn,
     # 10) deterministic_cudnn and 11) user_enabled_cudnn.
-    assert len(inputs) == 12, len(inputs)
+    # starting with #40737 it will be 12) user_enabled_tf32
+    assert len(inputs) == 12 or len(inputs) == 13, len(inputs)
     x, w = inputs[:2]
     x_shape, w_shape, out_shape = (get_shape(x), get_shape(w), get_shape(outputs[0]))
     return conv_flop_count(x_shape, w_shape, out_shape)
