@@ -1,6 +1,7 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 
 import inspect
+import pprint
 from abc import ABCMeta, abstractmethod
 from typing import Any, Callable, List, Optional, TypeVar
 
@@ -211,27 +212,70 @@ class Transform(metaclass=ABCMeta):
         """
         raise NotImplementedError
 
+    def __repr__(self):
+        """
+        Produce something like:
+        "MyTransform(field1={self.field1}, field2={self.field2})"
+        """
+        try:
+            sig = inspect.signature(self.__init__)
+            classname = type(self).__name__
+            argstr = []
+            for name, param in sig.parameters.items():
+                assert (
+                    param.kind != param.VAR_POSITIONAL
+                    and param.kind != param.VAR_KEYWORD
+                ), "The default __repr__ doesn't support *args or **kwargs"
+                assert hasattr(self, name), (
+                    "Attribute {} not found! "
+                    "Default __repr__ only works if attributes match the constructor.".format(
+                        name
+                    )
+                )
+                attr = getattr(self, name)
+                default = param.default
+                if default is attr:
+                    continue
+                attr_str = pprint.pformat(attr)
+                if "\n" in attr_str:
+                    # don't show it if pformat decides to use >1 lines
+                    attr_str = "..."
+                argstr.append("{}={}".format(name, attr_str))
+            return "{}({})".format(classname, ", ".join(argstr))
+        except AssertionError:
+            return super().__repr__()
+
 
 _T = TypeVar("_T")
 
 
 # pyre-ignore-all-errors
-class TransformList:
+class TransformList(Transform):
     """
     Maintain a list of transform operations which will be applied in sequence.
     Attributes:
         transforms (list[Transform])
     """
 
-    def __init__(self, transforms: list):
+    def __init__(self, transforms: List[Transform]):
         """
         Args:
             transforms (list[Transform]): list of transforms to perform.
         """
         super().__init__()
+        # "Flatten" the list so that TransformList do not recursively contain TransfomList.
+        # The additional hierarchy does not change semantic of the class, but cause extra
+        # complexities in e.g, telling whether a TransformList contains certain Transform
+        tfms_flatten = []
         for t in transforms:
-            assert isinstance(t, Transform), t
-        self.transforms = transforms
+            assert isinstance(
+                t, Transform
+            ), f"TransformList requires a list of Transform. Got type {type(t)}!"
+            if isinstance(t, TransformList):
+                tfms_flatten.extend(t.transforms)
+            else:
+                tfms_flatten.append(t)
+        self.transforms = tfms_flatten
 
     def _apply(self, x: _T, meth: str) -> _T:
         """
@@ -299,6 +343,20 @@ class TransformList:
         """
         return TransformList([x.inverse() for x in self.transforms[::-1]])
 
+    def __repr__(self) -> str:
+        msgs = [str(t) for t in self.transforms]
+        return "TransformList[{}]".format(", ".join(msgs))
+
+    __str__ = __repr__
+
+    # The actual implementations are provided in __getattribute__.
+    # But abstract methods need to be declared here.
+    def apply_coords(self, x):
+        raise NotImplementedError
+
+    def apply_image(self, x):
+        raise NotImplementedError
+
 
 class HFlipTransform(Transform):
     """
